import { Assert } from '@overflow/commons/core/util';
import { ScopeType, InjectModeType } from '@overflow/commons/di/type';

export class InstanceDefinition {
  private readonly attributes: Map<string, any> = new Map();
  private scope: ScopeType = ScopeType.Default;
  private injectMode: InjectModeType = InjectModeType.No;

  // Attributes

  public setAttribute(name: string, value?: any): void {
    Assert.notNull(name, 'Name must not be null');
    if (value !== undefined) {
      this.attributes.set(name, value);
    } else {
      this.removeAttribute(name);
    }
  }

  public getAttribute(name: string): any | undefined {
    Assert.notNull(name, 'Name must not be null');
    return this.attributes.get(name);
  }

  public removeAttribute(name: string): boolean {
    Assert.notNull(name, 'Name must not be null');
    return this.attributes.delete(name);
  }

  public hasAttribute(name: string): boolean {
    Assert.notNull(name, 'Name must not be null');
    return this.attributes.has(name);
  }

  public attributeNames(): IterableIterator<string> {
    return this.attributes.keys();
  }

  // Metadata
  public getMetadata(): AnnotationMetadata {
    return undefined;
  }

  public setScope(scope: ScopeType): void {
    this.scope = scope;
  }

  public getScope(): ScopeType {
    return this.scope;
  }

  public isSingleton(): boolean {
    return ScopeType.Singleton === this.scope || ScopeType.Default === this.scope;
  }

  public isTransiant(): boolean {
    return ScopeType.Transiant === this.scope;
  }

  public setInjectMode(injectMode: InjectModeType): void {
    this.injectMode = injectMode;
  }

  public getInjectMode(): InjectModeType {
    return this.injectMode;
  }

}
